import os
import re
import json
import requests

import logging
logging.basicConfig(level=logging.INFO)

from flask import Flask, request, Response, jsonify

from pythonping import ping

from utils.Make_request.make_request import make_request

from flasgger import Swagger, swag_from

servicepatterns = {
    'device': ['device', 'device_service', 'services'],
    'services': ['device', 'device_service', 'services'],
    'analytics': ['analytics_service', 'Analytics', 'analytics'],
    'event': ['event_service', 'event', 'Events', 'Event'],
    'events': ['event_service', 'event', 'Events', 'Event'],
    'image': ['imaging_service', 'image_service', 'Imaging', 'imaging', 'image_services'],
    'imaging': ['imaging_service', 'image_service', 'Imaging', 'imaging', 'image_services'],
    'deviceio': ['deviceIO_service', 'deviceio_service', 'deviceio'],
    'media': ['media_service', 'Media', 'media'],
    'ptz': ['ptz', 'ptz_service', 'PTZ', 'ptz_services'],
    'ptz_services': ['ptz_services', 'ptz', 'ptz_service', 'PTZ'],
    'onvif_ext': ['onvif_ext'],
    'Subscription': ['Subscription'],
    'scdl_service': ['scdl_service']
}

def get_replace_template(ipport, host_ip, octet = None, port = None):
    if port == None and octet == None:
        to_replace = r'http://{}/'.format(host_ip, octet)
    elif port == None:
        to_replace = r'http://{}/{}/'.format(host_ip, octet)
    elif port != None and octet == None:
        to_replace = r'http://{}:{}/'.format(host_ip, port)
    else:
        to_replace = r'http://{}:{}/{}/'.format(host_ip, port, octet)
    dict = {
        r'{}'.format(ipport): to_replace,
        r'http://255.255.255.255:80/': to_replace
    }
    return dict


'''
dic is dict where strings stored in template "what_replace: replace_with"
This function find all what_replace-strings in text and replace them with replace_with-strings
then return updated string
'''
def replace_all(text, dic):
    for i, j in dic.items():
        text = text.replace(i, j)
    return text

'''

'''
def get_response(service, Onvif, body, octet = None):
    if len(service) <= 30:
        check_connection = str(ping(Onvif.ip, timeout = 2, count = 1))
        if check_connection.find('Request timed out') == -1:
            response = Onvif.make_request(service, body)
            if response != '':
                if response.status_code != 500:
                    if response.status_code == 400 or response.text == '' or response.text.find('400 Bad Request') > 0 or response.text.find('404 Not Found') > 0 or response.text.find('404 File Not Found') > 0:
                        service = service.split('_')[0].lower()
                        if service.rfind('/') > -1:
                            to_list = service.split('/')
                            service = to_list[len(to_list) - 1].lower()
                        if service in list(servicepatterns.keys()):
                            service_length = len(servicepatterns[service])
                            iterrator = 0
                            while (response.status_code == 400 or response.text == '' or response.text.find('400 Bad Request') > 0 or response.text.find('404 Not Found') > 0 or response.text.find('404 File Not Found') > 0) and iterrator < service_length:
                                response = Onvif.make_request(servicepatterns[service][iterrator], body)
                                iterrator += 1
                        else:
                            logging.info('Нет сервиса {}'.format(service))
                    if response.status_code == 400 or response.text == '' or response.text.find('400 Bad Request') > 0 or response.text.find('404 Not Found') > 0 or response.text.find('404 File Not Found') > 0:
                        return ''
                    else:
                        return response.text
        else:
            return ''

'''

'''
def replace_token(response, octet):
    if response == None:
        return ''
    else:
        token_as = [m.start() for m in re.finditer('token=', response)]
        token_arrow = [m.start() for m in re.finditer('Token>', response)]
        token_to_replace_as = []
        token_to_replace_arrow = []
        for i in token_as:
            token = response[i + 7:]
            token = token[: token.find('"')]
            token_to_replace_as.append(token)
        for i in range(len(token_arrow)):
            if i % 2 == 1:
                pass
            else:
                token = response[token_arrow[i] + 5:]
                token = token[: token.find('<')]
                if '<' in token:
                    token = token.replace('<', '')
                elif '>' in token:
                    token = token.replace('>', '')
                token_to_replace_arrow.append(token)
        token_to_replace_as = [*set(token_to_replace_as)]
        token_to_replace_arrow = [*set(token_to_replace_arrow)]
        split = response.find('Body') + 5
        first_part = response[:split]
        second_part = response[split:]
        response = first_part
        if len(token_to_replace_as) > 0:
            for what_replace in token_to_replace_as:
                if what_replace != '':
                    token_what_replace = 'token="' + what_replace + '"'
                    to_replace = 'token="' + octet + '_' + what_replace + '"'
                    second_part = second_part.replace(token_what_replace, to_replace)
        if len(token_to_replace_arrow) > 0:
            for what_replace in token_to_replace_arrow:
                if what_replace != '':
                    token_what_replace = 'Token>' + what_replace + '<'
                    to_replace = 'Token>' + octet + '_' + what_replace + '<'
                    second_part = second_part.replace(token_what_replace, to_replace)
        response = response + second_part
        return response

def return_soap(response):
    return Response(response, mimetype='application/soap+xml')

def return_ports_and_presets(input):
    first = int(input["ports"]["first"])
    presets = {}
    for i in list(input["remote_controller"].keys()):
        preset = {}
        for j in list(input["remote_controller"][i].keys()):
            preset[j] = int(j) + first - 1
        presets[i] = preset
    #output["presets"] = presets
    return presets

HTTP_METHODS = ['GET', 'HEAD', 'POST', 'PUT', 'DELETE', 'CONNECT', 'OPTIONS', 'TRACE', 'PATCH']

#cameras = json.load(open(os.path.join(os.path.dirname(__file__), 'conf.json')))

class Proccesser():
    def __init__(self, name, ip, port, conf_path, cam_num):
        self.app = Flask(name)
        self.conf_path = conf_path
        self.ip = ip
        self.port = port
        self.cameras = json.load(open(conf_path))
        self.cam_num = cam_num

        swagger = Swagger(self.app, template_file=os.path.join(os.path.dirname(os.path.realpath(__file__)), 'swagger.yml'))

        @self.app.route("/image/<path>/", methods=HTTP_METHODS)
        def return_image(path):
            with open(os.path.join(os.path.dirname(os.path.realpath(__file__)), 'image', path), "rb") as f:
                return Response(f.read(), mimetype="image/jpeg")

        @self.app.route("/mains", methods=HTTP_METHODS)
        def return_mains():
            output = {}
            confs_directory = os.path.join(os.path.dirname(os.path.dirname(os.path.realpath(__file__))), "confs")
            for root, dirs, files in os.walk(os.path.abspath(confs_directory)):
                for file in files:
                    output[file] = json.load(open(os.path.join(root, file)))["ports"]["first"]
            return jsonify(output)

        @self.app.route("/inconf/all", methods=HTTP_METHODS)
        def return_all_conf_presets():
            output = {}
            confs_directory = os.path.join(os.path.dirname(os.path.dirname(os.path.realpath(__file__))), "confs")
            for root, dirs, files in os.walk(os.path.abspath(confs_directory)):
                for file in files:
                    output[file] = return_ports_and_presets(json.load(open(os.path.join(root, file))))
            return jsonify(output)

        @self.app.route("/inconf", methods=HTTP_METHODS)
        def return_this_conf_presets():
            return jsonify(return_ports_and_presets(self.cameras))

        @self.app.route("/preset/total", methods=HTTP_METHODS)
        @self.app.route("/preset/total/<preset>", methods=HTTP_METHODS)
        def return_total(preset = None):
            if preset == None:
                return jsonify({"number": len(list(self.cameras["remote_controller"].keys()))})
            else:
                return jsonify({"number": len(list(self.cameras["remote_controller"][preset].keys()))})

        @self.app.route("/ports", methods=HTTP_METHODS)
        def return_ports():
            return jsonify(self.cameras["ports"])

        # change preset when onvif-proxy instances are run by docker-compose
        @self.app.route("/preset/<location>", methods=HTTP_METHODS)
        def change_location(location):
            self.cameras["default"] = location
            ok = True
            for i in range(int(self.cameras["ports"]["first"]), int(self.cameras["ports"]["last"]) + 1):
                if i != int(self.port):
                    response = requests.get('http://{}:{}/update/{}'.format(self.ip, i, location), timeout=5)
                    logging.info("{} - result:{}".format(i, response.status_code))
                    if response.status_code != 200:
                        ok = False
            if ok == True:
                return Response(status=200)
            else:
                return Response(status=500)

        #change preset when onvif-proxy instances are run by systemd
        @self.app.route("/location/<preset>", methods=HTTP_METHODS)
        def change_preset(preset):
            if preset not in list(self.cameras["remote_controller"].keys()):
                return Response(status = 400)
            else:
                self.cameras["default"] = preset
                with open(self.conf_path, 'w') as j:
                    json.dump(self.cameras, j)
                return jsonify({"number": len(list(self.cameras["remote_controller"][self.cameras["default"]].keys()))})

        @self.app.route("/update/<location>", methods=HTTP_METHODS)
        def update_instance(location):
            self.cameras["default"] = location
            return Response(status=200)

        @self.app.route("/<number>/onvif/device_service", methods=HTTP_METHODS)
        @self.app.route("/<number>/onvif/services", methods=HTTP_METHODS)
        @self.app.route("/<number>/onvif/analytics", methods=HTTP_METHODS)
        @self.app.route("/<number>/onvif/event", methods=HTTP_METHODS)
        @self.app.route("/<number>/onvif/imaging", methods=HTTP_METHODS)
        @self.app.route("/<number>/onvif/media", methods=HTTP_METHODS)
        @self.app.route("/<number>/onvif/ptz", methods=HTTP_METHODS)
        @self.app.route("/<number>/onvif/onvif_ext", methods=HTTP_METHODS)
        @self.app.route("/<number>/onvif/analytics_service", methods=HTTP_METHODS)
        @self.app.route("/<number>/onvif/event_service", methods=HTTP_METHODS)
        @self.app.route("/<number>/onvif/image_service", methods=HTTP_METHODS)
        @self.app.route("/<number>/onvif/imaging_service", methods=HTTP_METHODS)
        @self.app.route("/<number>/onvif/media_service", methods=HTTP_METHODS)
        @self.app.route("/<number>/onvif/image_services", methods=HTTP_METHODS)
        @self.app.route("/<number>/onvif/ptz_services", methods=HTTP_METHODS)
        @self.app.route("/<number>/onvif/ptz_service", methods=HTTP_METHODS)
        @self.app.route("/<number>/onvif/Analytics", methods=HTTP_METHODS)
        @self.app.route("/<number>/onvif/Event", methods=HTTP_METHODS)
        @self.app.route("/<number>/onvif/Events", methods=HTTP_METHODS)
        @self.app.route("/<number>/onvif/Imaging", methods=HTTP_METHODS)
        @self.app.route("/<number>/onvif/Media", methods=HTTP_METHODS)
        @self.app.route("/<number>/onvif/Subscription", methods=HTTP_METHODS)
        @self.app.route("/<number>/onvif/PTZ", methods=HTTP_METHODS)
        def auditory(number):
            logging.info('Аудитория {}'.format(number))

            auditory = str(number)

            cameras = self.cameras

            ips = list(cameras['auditories'][auditory].keys())

            ip = cameras['auditories'][auditory][ips[0]]['ip']
            port = cameras['auditories'][auditory][ips[0]]['port']

            ipport = 'http://{}:{}/'.format(ip, port)

            body = request.data.decode()

            #logging.info('Request {}'.format(body))

            Onvif = make_request(ip, port)

            if body.find('GetCapabilities') > 0 or body.find('GetServices') > 0:
                response = get_response('device_service', Onvif, body)
                #logging.info('Capabilities {}'.format(response))
                response = replace_all(response,
                                       get_replace_template(ipport,
                                                            self.ip,#cameras['host_ip'],
                                                            auditory,
                                                            self.port))#cameras['port']))
                response = replace_all(response,
                                       get_replace_template(r'http://{}/'.format(ip),
                                                            self.ip,#cameras['host_ip'],
                                                            auditory,
                                                            self.port))#cameras['port']))
                response = response.encode()
            else:
                header = str(request.headers.get('Content-Type'))
                logging.info('Хедер {}'.format(header))
                action = header[header.find('action') + 8:]
                action = action[:action.find('"')]
                service = action[action.find('ver10') + 6: action.find('wsdl') - 1]
                logging.info('Сервис {}'.format(service))

                if body.find('GetScopes') > 0:
                    response = get_response(service, Onvif, body)

                    #logging.info('До Response {}'.format(response))

                    if response.find('/location/') > 0 and response.find('/location/') != response.find(
                            '/location/country'):
                        response = response.replace('/location/', '/location/{}'.format(auditory))
                    elif response.find(':location:') > 0:
                        location = response[response.find(':location:') + 10:]
                        location = location[:location.find('<')]
                        response = response.replace(location, '{}'.format(auditory))
                    elif response.find('/location/') > 0:
                        location = response[response.find('/location/') + 10:]
                        location = location[:location.find('<')]
                        response = response.replace(location, '{}'.format(auditory))

                elif body.find('GetProfiles') > 0:
                    response = get_response(service, Onvif, body)
                    response = replace_token(response, ips[0])
                    first_part = response[:response.rfind('GetProfilesResponse')]
                    first_part = first_part[:first_part.rfind('>') + 1]
                    second_part = response[response.rfind('Profiles>'):]
                    second_part = second_part[second_part.find('<'):]
                    response = first_part
                    for i in range(1, len(ips)):
                        Onvif = make_request(cameras['auditories'][auditory][ips[i]]['ip'],
                                             cameras['auditories'][auditory][ips[i]]['port'])
                        add_response = get_response(service, Onvif, body, ips[i])
                        add_response = replace_token(add_response, ips[i])
                        add_response = add_response[add_response.find('GetProfilesResponse') + 20:add_response.rfind(
                            'GetProfilesResponse')]
                        add_response = add_response[:add_response.rfind('>') + 1]
                        response = response + add_response
                    response = response + second_part
                    #logging.info('Response {}'.format(response))
                elif body.find('GetVideoSources') > 0:
                    response = get_response(service, Onvif, body)
                    response = replace_token(response, ips[0])
                    first_part = response[:response.rfind('GetVideoSourcesResponse')]
                    first_part = first_part[:first_part.rfind('>') + 1]
                    second_part = response[response.rfind('VideoSources>'):]
                    second_part = second_part[second_part.find('<'):]
                    response = first_part
                    for i in range(1, len(ips)):
                        Onvif = make_request(cameras['auditories'][auditory][ips[i]]['ip'],
                                             cameras['auditories'][auditory][ips[i]]['port'])

                        add_response = get_response(service, Onvif, body, ips[i])

                        add_response = replace_token(add_response, ips[i])
                        add_response = add_response[
                                       add_response.find('GetVideoSourcesResponse') + 24:add_response.rfind(
                                           'GetVideoSourcesResponse')]
                        add_response = add_response[:add_response.rfind('>') + 1]
                        response = response + add_response
                    response = response + second_part
                    response = re.sub("^\s+|\n|\r|\s+$", '', response)
                elif (body.rfind('Token') - 8) != body.rfind('UsernameToken') and body.rfind(
                        'Token') > -1 and body.rfind(
                        'UsernameToken') > -1:
                    token_with_ip = body[body.rfind('Token>', 0, body.rfind('Token>') - 7) + 6:]
                    token_with_ip = token_with_ip[: token_with_ip.find('<')]

                    octet = token_with_ip.split('_')[0]

                    token_without_ip = token_with_ip[len(octet) + 1:]

                    body = body.replace(token_with_ip, token_without_ip)
                    if body.find('GetVideoEncoderConfigurationOptions') > 0 or body.find(
                            'GetMetadataConfigurationOptions') > 0:
                        if body.find('ConfigurationToken>') > 0:
                            token_with_ip = body[body.find('ConfigurationToken>') + 19:]
                            token_with_ip = token_with_ip[: token_with_ip.find('<')]
                            token_without_ip = token_with_ip[len(octet) + 1:]
                            body = body.replace(token_with_ip, token_without_ip)

                    Onvif = make_request(cameras['auditories'][auditory][octet]['ip'],
                                         cameras['auditories'][auditory][octet]['port'])

                    response = get_response(service, Onvif, body)

                    if body.find('GetStreamUri') > 0:
                        old_streamuri = response[response.find('>rtsp') + 1:]
                        old_streamuri = old_streamuri[: old_streamuri.find('<')]
                        streamuri = cameras['auditories'][auditory][octet]['streamuri']
                        response = response.replace(old_streamuri, streamuri)
                    response = replace_token(response, octet)

                    if body.find('GetSnapshotUri'):
                        if response.find('255.255.255.255:80'):
                            response = replace_all(response, get_replace_template(ipport,
                                                                                  cameras['auditories'][auditory][
                                                                                      octet]['ip']))
                else:
                    response = get_response(service, Onvif, body)
                    response = replace_token(response, ips[0])
                response = re.sub("^\s+|\n|\r|\s+$", '', response)

            #logging.info('Итог Response {}'.format(response))
            return return_soap(response)

        @self.app.route("/onvif/device_service", methods=HTTP_METHODS)
        @self.app.route("/onvif/analytics", methods=HTTP_METHODS)
        @self.app.route("/onvif/services", methods=HTTP_METHODS)
        @self.app.route("/onvif/event", methods=HTTP_METHODS)
        @self.app.route("/onvif/imaging", methods=HTTP_METHODS)
        @self.app.route("/onvif/media", methods=HTTP_METHODS)
        @self.app.route("/onvif/ptz", methods=HTTP_METHODS)
        @self.app.route("/onvif/onvif_ext", methods=HTTP_METHODS)
        @self.app.route("/onvif/analytics_service", methods=HTTP_METHODS)
        @self.app.route("/onvif/event_service", methods=HTTP_METHODS)
        @self.app.route("/onvif/image_service", methods=HTTP_METHODS)
        @self.app.route("/onvif/imaging_service", methods=HTTP_METHODS)
        @self.app.route("/onvif/media_service", methods=HTTP_METHODS)
        @self.app.route("/onvif/ptz_service", methods=HTTP_METHODS)
        @self.app.route("/onvif/Analytics", methods=HTTP_METHODS)
        @self.app.route("/onvif/Event", methods=HTTP_METHODS)
        @self.app.route("/onvif/Events", methods=HTTP_METHODS)
        @self.app.route("/onvif/Imaging", methods=HTTP_METHODS)
        @self.app.route("/onvif/Media", methods=HTTP_METHODS)
        @self.app.route("/onvif/Subscription", methods=HTTP_METHODS)
        @self.app.route("/onvif/PTZ", methods=HTTP_METHODS)
        @self.app.route("/onvif/image_services", methods=HTTP_METHODS)
        @self.app.route("/onvif/ptz_services", methods=HTTP_METHODS)
        def remote_controller():

            #self.cameras = json.load(open(self.conf_path))

            paradigm = "remote_controller"
            cameras = self.cameras
            auditory = str(cameras["default"])
            cam_num = self.cam_num

            logging.info('Аудитория {}'.format(auditory))
            logging.info('Камера {}'.format(cam_num))

            ips = list(cameras[paradigm][auditory].keys())
            if int(cam_num) > 0:
                needed_ip = cam_num
            else:
                needed_ip = ips[0]
            ip = cameras[paradigm][auditory][needed_ip]['ip']
            port = cameras[paradigm][auditory][needed_ip]['port']

            ipport = 'http://{}:{}/'.format(ip, port)

            body = request.data.decode()

            Onvif = make_request(ip, port)

            if body.find('GetCapabilities') > 0 or body.find('GetServices') > 0:
                response = get_response('device_service', Onvif, body)
                response = replace_all(response, get_replace_template(ipport, self.ip, None, self.port))
                response = replace_all(response, get_replace_template(r'http://{}/'.format(ip), self.ip, None, self.port))
                response = response.encode()
            else:
                header = str(request.headers.get('Content-Type'))
                logging.info('Хедер {}'.format(header))
                action = header[header.find('action') + 8:]
                action = action[:action.find('"')]
                if action.find("ver10") > 0:
                    service = action[action.find('ver10') + 6: action.find('wsdl') - 1]
                elif action.find("ver20") > 0:
                    service = action[action.find('ver20') + 6: action.find('wsdl') - 1]
                logging.info('Сервис {}'.format(service))

                logging.info('Запрос {}'.format(body))

                if body.find('GetScopes') > 0:
                    response = get_response(service, Onvif, body)

                    if response.find('/location/') > 0 and response.find('/location/') != response.find('/location/country'):
                        response = response.replace('/location/', '/location/{}'.format(auditory))
                    elif response.find(':location:') > 0:
                        location = response[response.find(':location:') + 10:]
                        location = location[:location.find('<')]
                        response = response.replace(location, '{}'.format(auditory))
                    elif response.find('/location/') > 0:
                        location = response[response.find('/location/') + 10:]
                        location = location[:location.find('<')]
                        response = response.replace(location, '{}'.format(auditory))

                elif body.find('GetProfiles') > 0:
                    response = get_response(service, Onvif, body)
                    response = replace_token(response, needed_ip)
                    if int(cam_num) < 1:
                        first_part = response[:response.rfind('GetProfilesResponse')]
                        first_part = first_part[:first_part.rfind('>') + 1]
                        second_part = response[response.rfind('Profiles>'):]
                        second_part = second_part[second_part.find('<'):]
                        response = first_part
                        for i in range(1, len(ips)):
                            Onvif = make_request(cameras[paradigm][auditory][ips[i]]['ip'],
                                                 cameras[paradigm][auditory][ips[i]]['port'])
                            add_response = get_response(service, Onvif, body, ips[i])
                            add_response = replace_token(add_response, ips[i])
                            add_response = add_response[
                                           add_response.find('GetProfilesResponse') + 20:add_response.rfind(
                                               'GetProfilesResponse')]
                            add_response = add_response[:add_response.rfind('>') + 1]
                            response = response + add_response
                        response = response + second_part
                elif body.find('GetVideoSources') > 0:
                    response = get_response(service, Onvif, body)
                    response = replace_token(response, needed_ip)
                    if int(cam_num) < 1:
                        first_part = response[:response.rfind('GetVideoSourcesResponse')]
                        first_part = first_part[:first_part.rfind('>') + 1]
                        second_part = response[response.rfind('VideoSources>'):]
                        second_part = second_part[second_part.find('<'):]
                        response = first_part
                        for i in range(1, len(ips)):
                            Onvif = make_request(cameras[paradigm][auditory][ips[i]]['ip'],
                                                 cameras[paradigm][auditory][ips[i]]['port'])

                            add_response = get_response(service, Onvif, body, ips[i])
                            add_response = replace_token(add_response, ips[i])
                            add_response = add_response[
                                           add_response.find('GetVideoSourcesResponse') + 24:add_response.rfind(
                                               'GetVideoSourcesResponse')]
                            add_response = add_response[:add_response.rfind('>') + 1]

                            response = response + add_response
                        response = response + second_part
                        response = re.sub("^\s+|\n|\r|\s+$", '', response)

                elif (body.rfind('Token') - 8) != body.rfind('UsernameToken') and body.rfind(
                        'Token') > -1 and body.rfind('UsernameToken') > -1:

                    token_with_ip = body[body.rfind('Token>', 0, body.rfind('Token>') - 7) + 6:]
                    token_with_ip = token_with_ip[: token_with_ip.find('<')]
                    octet = token_with_ip.split('_')[0]

                    token_without_ip = token_with_ip[len(octet) + 1:]
                    body = body.replace(token_with_ip, token_without_ip)
                    if body.find('GetVideoEncoderConfigurationOptions') > 0 or body.find(
                            'GetMetadataConfigurationOptions') > 0:
                        if body.find('ConfigurationToken>') > 0:
                            token_with_ip = body[body.find('ConfigurationToken>') + 19:]
                            token_with_ip = token_with_ip[: token_with_ip.find('<')]
                            token_without_ip = token_with_ip[len(octet) + 1:]
                            body = body.replace(token_with_ip, token_without_ip)

                    Onvif = make_request(cameras[paradigm][auditory][octet]['ip'],
                                         cameras[paradigm][auditory][octet]['port'])

                    logging.info('Запрос к камере {}'.format(body))
                    response = get_response(service, Onvif, body)
                    logging.info('Ответ до переработки {}'.format(response))

                    if body.find('GetStreamUri') > 0:
                        old_streamuri = response[response.find('>rtsp') + 1:]
                        old_streamuri = old_streamuri[: old_streamuri.find('<')]
                        streamuri = cameras[paradigm][auditory][octet]['streamuri']
                        response = response.replace(old_streamuri, streamuri)
                    response = replace_token(response, octet)

                    if body.find('GetSnapshotUri'):
                        if response.find('255.255.255.255:80'):
                            response = replace_all(response, get_replace_template(ipport,
                                                                                  cameras[paradigm][auditory][octet][
                                                                                      'ip']))
                else:
                    response = get_response(service, Onvif, body)
                    response = replace_token(response, ips[0])
                logging.info('Ответ после обработки {}'.format(response))
                response = re.sub("^\s+|\n|\r|\s+$", '', response)
            return return_soap(response)

    def run(self):
        self.app.run(host='0.0.0.0', port=int(self.port), debug=True)
